%bcond_without gnutls
%bcond_without openssl
%bcond_with picotls
%bcond_with boringssl

%bcond_without openssl_external

%if %{with openssl}
%global __brp_check_rpaths %{nil}
%endif

Summary: ngtcp2 project is an effort to implement RFC9000 QUIC protocol.
Name: ngtcp2
Version: 0.12.0
Release: 1%{?dist}
License: MIT
URL: https://nghttp2.org/ngtcp2/
Source0: https://github.com/ngtcp2/ngtcp2/releases/download/v%{version}/ngtcp2-%{version}.tar.gz
%if %{with openssl}
%if %{without openssl_external}
Source1: openssl-3.0.2+quic.tar.gz
%endif
%endif
%if %{with boringssl}
Source2: boringssl.tar.gz
%endif
%if %{with picotls}
Source3: picotls-0.0.1.tar.gz
%endif
BuildRequires: CUnit-devel
BuildRequires: cmake
BuildRequires: git
BuildRequires: cmake-rpm-macros
BuildRequires: gcc-c++
BuildRequires: make
BuildRequires: openssl-devel
BuildRequires: libev-devel
BuildRequires: libnghttp3-devel >= 0.4.0
BuildRequires: nettle-devel
%if %{with openssl}
%if %{with openssl_external}
BuildRequires: openssl-quic-devel >= 3.0.0
Requires:      openssl-quic-libs >= 3.0.0
%endif
%endif
Requires: libev
%if %{with gnutls}
BuildRequires: gnutls-devel
%endif
%if %{with openssl}
BuildRequires: perl(Test::Harness), perl(Test::More), perl(Math::BigInt)
BuildRequires: perl(Module::Load::Conditional), perl(File::Temp)
BuildRequires: perl(Time::HiRes), perl(IPC::Cmd), perl(Pod::Html), perl(Digest::SHA)
BuildRequires: perl(FindBin), perl(lib), perl(File::Compare), perl(File::Copy), perl(bigint)
%endif

Requires: libngtcp2%{?_isa} = %{version}-%{release}
%{?systemd_requires}

%description
ngtcp2 project is an effort to implement RFC9000 QUIC protocol.


%package -n libngtcp2
Summary: A library implementing the RFC9000 QUIC protocol.

%description -n libngtcp2
libngtcp2 is a library  RFC9000 QUIC protocol.
version 2 (HTTP/3) protocol in C.

%package -n libngtcp2-static
Summary: A library implementing the RFC9000 QUIC protocol.

%description -n libngtcp2-static
libngtcp2 is a library  RFC9000 QUIC protocol.
version 2 (HTTP/3) protocol in C.


%package -n libngtcp2-devel
Summary: Files needed for building applications with libngtcp2
Requires: libngtcp2%{?_isa} = %{version}-%{release}
Requires: pkgconfig

%description -n libngtcp2-devel
The libngtcp2-devel package includes libraries and header files needed
for building applications with libngtcp2.


%prep
%setup -q

%if %{with openssl}
%if %{without openssl_external}
mkdir -p openssl
tar xzf %{SOURCE1} --strip-components 1 -C openssl
%endif
%endif

%if %{with boringssl}
mkdir -p boringssl
tar xzf %{SOURCE2} --strip-components 1 -C boringssl
%endif

%if %{with picotls}
mkdir -p picotls
tar xzf %{SOURCE3} --strip-components 1 -C picotls
%endif


%build

# Build PicoTLS
%if %{with picotls}
pushd picotls
mkdir -p build
pushd build
cmake -DCMAKE_POSITION_INDEPENDENT_CODE=ON ..

make %{?_smp_mflags}
# %  make_build
popd
popd

PICOTLS_CFLAGS="-I$PWD/picotls/include/"
PICOTLS_LIBS="-L$PWD/picotls/build -lpicotls-openssl -lpicotls-core"
export PICOTLS_CFLAGS
export PICOTLS_LIBS
%endif

# Build OpenSSL
%if %{with openssl}
%if %{without openssl_external}
pushd openssl
./config --prefix=$PWD/build

%make_build
%make_build install_sw
popd

OPENSSL_CFLAGS="-I$PWD/openssl/include/"
OPENSSL_LIBS="-L$PWD/openssl/build/ssl -lssl -L$PWD/openssl/build/crypto -lcrypto"
PKG_CONFIG_PATH="$PWD/openssl/build/lib/pkgconfig:$PWD/openssl/build/lib64/pkgconfig"
LDFLAGS="$EXTRA_LDFLAGS -Wl,-rpath,$PWD/openssl/build/lib -Wl,-rpath,$PWD/openssl/build/lib64"
export OPENSSL_CFLAGS
export OPENSSL_LIBS
export PKG_CONFIG_PATH
export LDFLAGS
%else
# Use internal OpenSSL
OPENSSL_CFLAGS="-I%{_prefix}/openssl-quic/include/"
OPENSSL_LIBS="-L%{_prefix}/openssl-quic/%{_lib} -lssl -lcrypto"
PKG_CONFIG_PATH="%{_prefix}/openssl-quic/%{_lib}/pkgconfig"
LDFLAGS="$EXTRA_LDFLAGS -Wl,-rpath,%{_prefix}/openssl-quic/%{_lib}"
export OPENSSL_CFLAGS
export OPENSSL_LIBS
export PKG_CONFIG_PATH
export LDFLAGS
%endif
%endif

# Build BoringSSL
%if %{with boringssl}
pushd boringssl
mkdir -p build
pushd build
cmake -DCMAKE_POSITION_INDEPENDENT_CODE=ON ..

%make_build
popd
popd

BORINGSSL_CFLAGS="-I$PWD/boringssl/include/"
BORINGSSL_LIBS="-L$PWD/boringssl/build/ssl -lssl -L$PWD/boringssl/build/crypto -lcrypto"
BORINGSSL_LIBS="$BORINGSSL_LIBS -pthread"
export BORINGSSL_CFLAGS
export BORINGSSL_LIBS
%endif


%cmake \
%if %{with gnutls}
    -DENABLE_GNUTLS=1 \
%else
    -DENABLE_GNUTLS=0 \
%endif
%if %{with boringssl}
    -DENABLE_BORINGSSL=1 \
    -DBORINGSSL_LIBRARIES="$BORINGSSL_LIBS" \
    -DBORINGSSL_INCLUDE_DIR="$PWD/boringssl/include/" \
%else
    -DENABLE_BORINGSSL=0 \
%endif
%if %{with picotls}
    -DENABLE_PICOTLS=ON \
    -DPICOTLS_LIBRARIES="$PICOTLS_LIBS" \
    -DPICOTLS_INCLUDE_DIR="$PWD/picotls/include/" \
%else
    -DENABLE_PICOTLS=0 \
%endif
%if %{with openssl}
%if %{without openssl_external}
    -DENABLE_OPENSSL=1 \
    -DOPENSSL_LIBRARIES="$OPENSSL_LIBS" \
    -DOPENSSL_INCLUDE_DIR="$PWD/openssl/include" \
%else
    -DENABLE_OPENSSL=1 \
    -DOPENSSL_LIBRARIES="$OPENSSL_LIBS" \
    -DOPENSSL_INCLUDE_DIR="%{_prefix}/openssl-quic/include" \
%endif
%endif

%cmake_build

%install
rm -rf %{buildroot}

%cmake_install

%files
%doc README.rst

%files -n libngtcp2
%{_libdir}/libngtcp2.so.*
%{!?_licensedir:%global license %%doc}
%license COPYING

%if %{with gnutls}
%{_libdir}/libngtcp2_crypto_gnutls.so.*
%endif

%if %{with openssl}
%{_libdir}/libngtcp2_crypto_openssl.so.*
%endif

%if %{with boringssl}
%{_libdir}/libngtcp2_crypto_boringssl.so.*
%endif

%files -n libngtcp2-static
%{_libdir}/libngtcp2_static.a
%{!?_licensedir:%global license %%doc}
%license COPYING

%if %{with gnutls}
%{_libdir}/libngtcp2_crypto_gnutls_static.so
%{_libdir}/libngtcp2_crypto_gnutls_static.so.*
%endif

%if %{with openssl}
%{_libdir}/libngtcp2_crypto_openssl_static.so
%{_libdir}/libngtcp2_crypto_openssl_static.so.*
%endif

%if %{with boringssl}
%{_libdir}/libngtcp2_crypto_boringssl_static.so
%{_libdir}/libngtcp2_crypto_boringssl_static.so.*
%endif

%files -n libngtcp2-devel
%{_includedir}/ngtcp2
%{_libdir}/libngtcp2.so
%{_libdir}/pkgconfig/libngtcp2.pc

%if %{with gnutls}
%{_libdir}/libngtcp2_crypto_gnutls.so
%{_libdir}/pkgconfig/libngtcp2_crypto_gnutls.pc
%endif

%if %{with openssl}
%{_libdir}/libngtcp2_crypto_openssl.so
%{_libdir}/pkgconfig/libngtcp2_crypto_openssl.pc
%endif

%if %{with boringssl}
%{_libdir}/libngtcp2_crypto_boringssl.so
%{_libdir}/pkgconfig/libngtcp2_crypto_boring.pc
%endif

%changelog
* Fri Dec 9 2022 Zoran Peričić <zoran.pericic@infomaas.com> 0.11.0-1
- v0.11.0

* Thr Jul 8 2022 Zoran Peričić <zoran.pericic@infomaas.com> 0.6.0-1
- v0.6.0

* Thu Apr 28 2022 Zoran Peričić <zoran.pericic@infomaas.com> 0.4.0-1
- v0.4.0

* Fri Apr 8 2022 Zoran Peričić <zoran.pericic@infomaas.com> 0.3.0-1
- v0.3.0

* Wed Dec 29 2021 Zoran Peričić <zoran.pericic@infomaas.com> 0.1.0-1
- Initial build
